/*
Pluck: an open source trivia game for Android

Copyright (C) 2021 Joguitos do luca0N!

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see <https://www.gnu.org/licenses/>.

This game is a fork of LibreTrivia, and its source code is available at
<https://github.com/tryton-vanmeer/LibreTrivia>.

Contact us at <joguitos+pluck@luca0n.com>.
*/

package com.luca0n.joguitos.pluck.activities;

import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Build;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.Spinner;
import androidx.appcompat.app.AlertDialog;

import butterknife.BindView;
import butterknife.ButterKnife;

import com.luca0n.joguitos.pluck.R;
import com.luca0n.joguitos.pluck.trivia.TriviaSource;
import com.luca0n.joguitos.pluck.trivia.TriviaCategory;
import com.luca0n.joguitos.pluck.trivia.TriviaDifficulty;
import com.luca0n.joguitos.pluck.trivia.TriviaQuery;
import com.luca0n.joguitos.pluck.trivia.TriviaFilters;

public class MainActivity extends BaseActivity {

	@BindView(R.id.button_play)
	Button buttonPlay;
	@BindView(R.id.spinner_source)
	Spinner spinnerSource;
	@BindView(R.id.spinner_number)
	Spinner spinnerNumber;
	@BindView(R.id.spinner_category)
	Spinner spinnerCategory;
	@BindView(R.id.spinner_difficulty)
	Spinner spinnerDifficulty;

	private static final int REQUEST_OPEN_DOCUMENT = 0;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		ButterKnife.bind(this);

		buttonPlay.setOnClickListener(v -> {
			int sourcePos = spinnerSource.getSelectedItemPosition();
			if (TriviaSource.values()[sourcePos] ==  TriviaSource.FILE){
				// Check the Android version.
				if (Build.VERSION.SDK_INT >= 19){
					// TODO: Show the open document dialog and read the trivia file.
					Intent i = new Intent(Intent.ACTION_OPEN_DOCUMENT);
					i.setType("*/*");
					startActivityForResult(i, REQUEST_OPEN_DOCUMENT);
					return;
				} else {
					// Unsupported version.
					new AlertDialog.Builder(this)
					.setIcon(android.R.drawable.ic_dialog_alert)
					.setTitle(R.string.error)
					.setMessage(R.string.error_source_file_android_version_unsupported)
					.setPositiveButton(android.R.string.ok, null)
					.show();	
				}
				return;
			}
			int amount = (int) spinnerNumber.getSelectedItem();
			TriviaCategory category = (TriviaCategory) spinnerCategory.getSelectedItem();
			TriviaDifficulty difficulty = (TriviaDifficulty) spinnerDifficulty.getSelectedItem();

			// Put the server address in the TriviaQuery object.	
			SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(getApplicationContext());
			String serverKey = getResources().getString(R.string.pref_network_server);
			// Load default server URL in case the player did not specify a custom server URL.
			String defaultServer = getResources().getString(R.string.pref_network_server_default);
			String base = sp.getString(serverKey, defaultServer) + "?"; // append "?" to the end of the URL so we can specify GET parameters.

			// Check if the custom server URL preference was empty. If it was, change it to the default URL.
			if (base.equals("?")){
				base = defaultServer + "?";
				sp.edit().putString(serverKey, defaultServer).commit();
			}

			TriviaQuery query = new TriviaQuery.Builder(amount)
				.category(category)
				.difficulty(difficulty)
				.base(base)
				.build();

			// Launch the trivia game activity.
			Intent intent = new Intent(getApplicationContext(), TriviaGameActivity.class);
			intent.putExtra(TriviaGameActivity.EXTRA_TRIVIA_SOURCE, TriviaSource.SERVER.getName()); // Specify the trivia source.
			intent.putExtra(TriviaGameActivity.EXTRA_TRIVIA_QUERY, query);
			intent.putExtra(TriviaGameActivity.EXTRA_TRIVIA_FILTERS,
				new TriviaFilters.Builder(amount)
					.category(category)
					.difficulty(difficulty)
					.build());
			startActivity(intent);
		});

		Integer[] numbers = new Integer[50];
		for (int i = 0; i < 50; ) {
			numbers[i] = ++i;
		}
		spinnerSource.setAdapter(
				new ArrayAdapter<>(
					this, android.R.layout.simple_list_item_1, TriviaSource.values())
				);
		spinnerNumber.setAdapter(
				new ArrayAdapter<>(
					this, android.R.layout.simple_list_item_1, numbers)
				);
		spinnerNumber.setSelection(9);

		spinnerCategory.setAdapter(
				new ArrayAdapter<>(
					this, android.R.layout.simple_list_item_1, TriviaCategory.values()));

		spinnerDifficulty.setAdapter(
				new ArrayAdapter<>(
					this, android.R.layout.simple_list_item_1, TriviaDifficulty.values()));
	}

	@Override
	public void onActivityResult(int requestCode, int resultCode, Intent extras){
		switch (requestCode){
			case REQUEST_OPEN_DOCUMENT:
				// Was the request executed successfully?
				if (resultCode == Activity.RESULT_OK){
					// Check if the user canceled the operation.
					if (extras == null)
						break;
	
					int amount = (int) spinnerNumber.getSelectedItem();
					TriviaCategory category = (TriviaCategory) spinnerCategory.getSelectedItem();
					TriviaDifficulty difficulty = (TriviaDifficulty) spinnerDifficulty.getSelectedItem();

					Intent intent = new Intent(getApplicationContext(), TriviaGameActivity.class);
					// Put filters on the extras object.
					intent.putExtra(TriviaGameActivity.EXTRA_TRIVIA_FILTERS,
							new TriviaFilters.Builder(amount)
								.category(category)
								.difficulty(difficulty)
								.build());
					intent.putExtra(TriviaGameActivity.EXTRA_TRIVIA_SOURCE, TriviaSource.FILE.getName()); // Specify the trivia source.
					intent.putExtra(TriviaGameActivity.EXTRA_TRIVIA_SOURCE_DATA, extras.getData()); // Specify the trivia source data. In this case, since the user chose the "File" source option, we should put the file Uri object here.
					startActivity(intent);
				} else {
					new AlertDialog.Builder(this)
						.setIcon(android.R.drawable.ic_dialog_alert)
						.setTitle(R.string.error)
						.setMessage(R.string.error_trivia_file_fetch_unknown)
						.setPositiveButton(android.R.string.ok, null)
						.show();
				}
				break;
			default:
				super.onActivityResult(requestCode, resultCode, extras);
				break;
		}
	}
}
