/*
Pluck: an open source trivia game for Android

Copyright (C) 2021 Joguitos do luca0N!

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see <https://www.gnu.org/licenses/>.

This game is a fork of LibreTrivia, and its source code is available at
<https://github.com/tryton-vanmeer/LibreTrivia>.

Contact us at <joguitos+pluck@luca0n.com>.
*/

package com.luca0n.joguitos.pluck.activities;

import android.content.Intent;
import android.content.res.ColorStateList;
import android.graphics.PorterDuff;
import android.graphics.PorterDuffColorFilter;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.net.Uri;
import android.view.View;
import android.view.MenuItem;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.io.Reader;
import java.io.InputStreamReader;
import java.io.InputStream;
import java.io.BufferedReader;

import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentTransaction;
import butterknife.BindView;
import butterknife.ButterKnife;
import com.luca0n.joguitos.pluck.R;
import com.luca0n.joguitos.pluck.exceptions.NoTriviaResultsException;
import com.luca0n.joguitos.pluck.exceptions.InvalidTriviaFormatException;
import com.luca0n.joguitos.pluck.fragments.TriviaGameErrorFragment;
import com.luca0n.joguitos.pluck.fragments.TriviaQuestionFragment;
import com.luca0n.joguitos.pluck.interfaces.IDownloadTriviaQuestionReceiver;
import com.luca0n.joguitos.pluck.trivia.TriviaGame;
import com.luca0n.joguitos.pluck.trivia.TriviaQuery;
import com.luca0n.joguitos.pluck.trivia.TriviaQuestion;
import com.luca0n.joguitos.pluck.trivia.TriviaSource;
import com.luca0n.joguitos.pluck.trivia.TriviaFilters;
import com.luca0n.joguitos.pluck.util.ApiUtil;
import com.luca0n.joguitos.pluck.util.SoundUtil;

public class TriviaGameActivity extends BaseActivity
	implements IDownloadTriviaQuestionReceiver {
	static final String EXTRA_TRIVIA_QUERY = "extra_trivia_query";
	static final String EXTRA_TRIVIA_SOURCE = "extra_trivia_source";
	static final String EXTRA_TRIVIA_SOURCE_DATA = "extra_trivia_source_data";
	static final String EXTRA_TRIVIA_FILTERS = "extra_trivia_filters";
	private final String STATE_TRIVIA_GAME = "state_trivia_game";

	private TriviaGame game;
	private TriviaSource source;
	private TriviaFilters filters;

	@BindView(R.id.progress_bar)
	ProgressBar progressBar;
	@BindView(R.id.trivia_status_bar)
	LinearLayout triviaStatusBar;
	@BindView(R.id.text_question_category)
	TextView textViewQuestionCategory;
	@BindView(R.id.text_question_difficulty)
	TextView textViewQuestionDifficulty;
	@BindView(R.id.text_question_progress)
	TextView textViewQuestionProgress;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_trivia_game);
		ButterKnife.bind(this);

		ActionBar actionBar = getSupportActionBar();
		if (actionBar != null)
			actionBar.setDisplayHomeAsUpEnabled(true);

		if (savedInstanceState != null) {
			this.game = (TriviaGame) savedInstanceState.getSerializable(STATE_TRIVIA_GAME);
		} else {
			Bundle bundle = getIntent().getExtras();
			assert bundle != null;
			TriviaQuery query = (TriviaQuery) bundle.get(EXTRA_TRIVIA_QUERY);
		
			filters = (TriviaFilters) bundle.get(EXTRA_TRIVIA_FILTERS);

			progressBar.setVisibility(View.VISIBLE);

			// Gather information about the trivia location.
			String sourceData;
			source = TriviaSource.get(bundle.getString(EXTRA_TRIVIA_SOURCE));
			switch (source){
				case SERVER:
					DownloadTriviaQuestionsTask task = new DownloadTriviaQuestionsTask();
					task.setReceiver(this);
					task.execute(query);
					break;
				case FILE:
					// Attempt to open the trivia file specified by the player.
					// Read the file as a plain text JSON trivia file.
					StringBuilder sb = new StringBuilder();
					try {
						InputStream is = getContentResolver().openInputStream((Uri) bundle.get(EXTRA_TRIVIA_SOURCE_DATA));
						Reader in = new BufferedReader(new InputStreamReader(is, StandardCharsets.UTF_8));
						for (int c; (c = in.read()) >= 0;)
							sb.append((char) c);
					} catch (IOException e){
						onException(e);
						return;
					}
					onTriviaQuestionsDownloaded(sb.toString());
					break;
				default:
					// Unknown trivia source.
					// TODO: Show error message to the user about this internal error.
					break;
			}
		}
	}

	@Override
	public void onSaveInstanceState(Bundle outState) {
		super.onSaveInstanceState(outState);
		outState.putSerializable(STATE_TRIVIA_GAME, this.game);
	}

	@Override
	public void onBackPressed() {
		Fragment fragment = getSupportFragmentManager().findFragmentById(R.id.frame_trivia_game);

		if (fragment instanceof TriviaGameErrorFragment) {
			super.onBackPressed();
		} else {
			new AlertDialog.Builder(this)
				.setTitle(R.string.ui_quit_game)
				.setMessage(R.string.ui_quit_game_msg)
				.setPositiveButton(android.R.string.yes, (dialog, which) ->
						TriviaGameActivity.super.onBackPressed())
				.setNegativeButton(android.R.string.no, (dialog, which) -> {
				})
			.show();
		}
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
			case android.R.id.home:
				finish();
				return true;
			default:
				return super.onOptionsItemSelected(item);
		}
	}

	public void onTriviaQuestionsDownloaded(String json) {
		if (json == null) {
			onNetworkError();
			return;
		} else {
			try {
				this.game = new TriviaGame(ApiUtil.jsonToQuestionArray(json, filters));
			} catch (Exception e) {
				onException(e);
				return;
			}
		}

		// Setup game layout
		progressBar.setVisibility(View.GONE);
		triviaStatusBar.setVisibility(View.VISIBLE);
		updateStatusBar();
		updateTriviaQuestion();
	}

	private void updateStatusBar() {
		String progress = getResources().getString(R.string.ui_question_progress,
				game.getQuestionProgress(), game.getQuestionsCount());

		String category = (game.getCurrentQuestion().getCategory() != null)
			? game.getCurrentQuestion().getCategory().toString() : "";

		String difficulty = game.getCurrentQuestion().getDifficulty().toString();

		textViewQuestionProgress.setText(progress);
		textViewQuestionCategory.setText(category);
		textViewQuestionDifficulty.setText(difficulty);
	}

	private void updateTriviaQuestion() {
		Fragment fragment = TriviaQuestionFragment.newInstance();
		getSupportFragmentManager().beginTransaction()
			.replace(R.id.frame_trivia_game, fragment)
			.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_OPEN)
			.commit();
	}

	private void onNetworkError() {
		String msg = getResources().getString(R.string.error_network);
		Fragment errorFragment = TriviaGameErrorFragment.newInstance(msg);

		// Hide the progress bar.
		progressBar.setVisibility(View.GONE);

		FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
		ft.replace(R.id.frame_trivia_game, errorFragment);
		ft.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_OPEN);
		ft.commit();
	}

	/**
	 * Handles caught exceptions and gives feedback to alert errors to the player.
	 * @param e The caught exception.
	 */
	private void onException(Exception e) {
		// Hide the progress bar.
		progressBar.setVisibility(View.GONE);
		String msg;
		int id;
		if (e instanceof NoTriviaResultsException)
			id = R.string.error_no_trivia_results;
		else if (e instanceof InvalidTriviaFormatException)
			// Check the trivia source type.
			if (source == TriviaSource.SERVER)
				id = R.string.error_server_response_invalid;
			else
				id = R.string.error_trivia_file_invalid;
		else 
			id = R.string.error_unknown;

		new AlertDialog.Builder(this)
			.setMessage(e.toString())
			.show();

		msg = getResources().getString(id);

		Fragment errorFragment = TriviaGameErrorFragment.newInstance(msg);

		FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
		ft.replace(R.id.frame_trivia_game, errorFragment);
		ft.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_OPEN);
		ft.commit();
	}

	public TriviaQuestion getCurrentQuestion() {
		return this.game.getCurrentQuestion();
	}

	public void onAnswerClick(Button answer, Button correctAnswer) {
		boolean guess = game.nextQuestion(answer.getText().toString());

		final int green = getResources().getColor(R.color.colorAccentGreen);
		int color = guess ? green
			: getResources().getColor(R.color.colorAccentRed);

		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
			ColorStateList stateList = ColorStateList.valueOf(color);
			answer.setBackgroundTintList(stateList);

			if (!guess) {
				final ColorStateList greenStateList = ColorStateList.valueOf(green);
				correctAnswer.setBackgroundTintList(greenStateList);
			}
		} else {
			answer.getBackground().getCurrent().setColorFilter(
					new PorterDuffColorFilter(color, PorterDuff.Mode.MULTIPLY));

			if (!guess)
				correctAnswer.getBackground().getCurrent().setColorFilter(
						new PorterDuffColorFilter(green, PorterDuff.Mode.MULTIPLY));
		}

		SoundUtil.playSound(this, guess ?
				SoundUtil.SOUND_ANSWER_CORRECT : SoundUtil.SOUND_ANSWER_WRONG);

		new Handler().postDelayed(() -> {
			if (game.isDone()) {
				Intent intent = new Intent(getApplicationContext(), TriviaGameResultsActivity.class);
				intent.putExtra(TriviaGameResultsActivity.EXTRA_TRIVIA_GAME, game);
				startActivity(intent);
				finish();
			} else {
				updateStatusBar();
				updateTriviaQuestion();
			}
		}, 500);
	}

	private static class DownloadTriviaQuestionsTask extends AsyncTask<TriviaQuery, Integer, String> {
		private IDownloadTriviaQuestionReceiver receiver;

		@Override
		protected String doInBackground(TriviaQuery... query) {
			String json;
			try {
				json = ApiUtil.GET(query[0]);
			} catch (IOException e) {
				return null;
			}
			return json;
		}

		@Override
		protected void onPostExecute(String json) {
			receiver.onTriviaQuestionsDownloaded(json);
		}

		private void setReceiver(IDownloadTriviaQuestionReceiver receiver) {
			this.receiver = receiver;
		}
	}
}
