/*
Pluck: an open source trivia game for Android

Copyright (C) 2021 Joguitos do luca0N!

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see <https://www.gnu.org/licenses/>.

This game is a fork of LibreTrivia, and its source code is available at
<https://github.com/tryton-vanmeer/LibreTrivia>.

Contact us at <joguitos+pluck@luca0n.com>.
*/

package com.luca0n.joguitos.pluck.util;

import android.content.Context;
import android.content.SharedPreferences;
import android.preference.PreferenceManager;

import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.google.gson.JsonSyntaxException;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.InetSocketAddress;
import java.net.Proxy;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;

import com.luca0n.joguitos.pluck.R;
import com.luca0n.joguitos.pluck.PluckApplication;
import com.luca0n.joguitos.pluck.exceptions.NoTriviaResultsException;
import com.luca0n.joguitos.pluck.exceptions.InvalidTriviaFormatException;
import com.luca0n.joguitos.pluck.trivia.TriviaQuery;
import com.luca0n.joguitos.pluck.trivia.TriviaQuestion;
import com.luca0n.joguitos.pluck.trivia.TriviaCategory;
import com.luca0n.joguitos.pluck.trivia.TriviaDifficulty;
import com.luca0n.joguitos.pluck.trivia.TriviaQuestionBoolean;
import com.luca0n.joguitos.pluck.trivia.TriviaQuestionMultiple;
import com.luca0n.joguitos.pluck.trivia.TriviaType;
import com.luca0n.joguitos.pluck.trivia.TriviaFilters;

public class ApiUtil {

	private static String readStream(InputStream in) throws IOException {
		StringBuilder builder = new StringBuilder();
		BufferedReader reader = new BufferedReader(new InputStreamReader(in), 1000);

		for (String line = reader.readLine(); line != null; line = reader.readLine()) {
			builder.append(line);
		}

		in.close();
		return builder.toString();
	}

	private static String GET(String query) throws IOException {
		String response;

		URL url = new URL(query);
		HttpURLConnection connection;

		// Check if the player wants to play the game using Tor.
		Context c = PluckApplication.getAppContext();
		SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(c);
		String key = c.getResources().getString(R.string.pref_network_tor);
		// Connect without Tor by default.
		boolean useTor = sp.getBoolean(key, false);
		// Default Orbot SOCKS5 address.
		String orbotSocks5Hostname = "127.0.0.1";
		int orbotSocks5Port = 9050;

		if (useTor)
			connection = (HttpURLConnection) url.openConnection(
					new Proxy(Proxy.Type.SOCKS, new InetSocketAddress(orbotSocks5Hostname, orbotSocks5Port)));
		else
			connection = (HttpURLConnection) url.openConnection();

		try {
			InputStream in = new BufferedInputStream(connection.getInputStream());
			response = readStream(in);
		} finally {
			connection.disconnect();
		}

		return response;
	}

	public static String GET(TriviaQuery query)
			throws IOException {
		return GET(query.toString());
	}

	public static ArrayList<TriviaQuestion> jsonToQuestionArray(String json,
			TriviaFilters filters) throws NoTriviaResultsException,
			InvalidTriviaFormatException {
		JsonObject jsonObject;
		try{
			jsonObject = new JsonParser().parse(json).getAsJsonObject();
		} catch (JsonSyntaxException e){
			// Unknown server response.
			throw new InvalidTriviaFormatException(e);
		}

		if (jsonObject.get("response_code").getAsInt() == 1) {
			throw new NoTriviaResultsException();
		}

		JsonArray jsonArray = jsonObject.getAsJsonArray("results");

		ArrayList<TriviaQuestion> questions = new ArrayList<>();

		for (JsonElement element : jsonArray) {
			JsonObject object = element.getAsJsonObject();
			TriviaType type = TriviaType.get(object.get("type").getAsString());

			TriviaCategory fc = filters.getCategory();

			// Check if this question matches the user specified criteria.
			if (fc != null &&
				fc != TriviaCategory.ANY && !object.get("category").getAsString().equals(fc.getName()))
					continue;

			if (filters.getDifficulty() != TriviaDifficulty.ANY && !object.get("difficulty").getAsString().equals(filters.getDifficulty().getName()))
				continue;

			if (type == TriviaType.MULTIPLE) {
				questions.add(TriviaQuestionMultiple.fromJson(object));
			} else {
				questions.add(TriviaQuestionBoolean.fromJson(object));
			}
		}

		// Check if there are no questions that match the specified user criteria.
		if (questions.size() == 0)
			throw new NoTriviaResultsException();

		// Shuffle the question array.
		Collections.shuffle(questions);

		return questions;
	}
}
